#!/usr/bin/env python
"""WSGI adaptors version 2 for integrating AuthKit with Django.

This module is based on the ``powerpack_adaptors.py`` extension, and
adds extra customizations for Django.

Copyright (C) 2010 Etienne Robillard <erob@gthcfoundation.org>
All rights reserved

<LICENSE>
"""

from decorator import decorator
from authkit.authorize import NotAuthenticatedError, NotAuthorizedError
from authkit.authorize import middleware as authkit_authorize_middleware

#Unused imports
#from authkit.authorize import authorize_request as authkit_authorize_request
#from authkit.authorize import PermissionSetupError

__all__ = ['authorize', 'authorize_request', 'authorized']

def authorize(permission):
    """
    This is a decorator which can be used to decorate a generic view.

    It takes the permission to check as the only argument and can be used with
    all types of permission objects.
    """
    def validate(func, request, *args, **kwargs):
        auth_conf = request.environ.get('authkit.config', None)
        if auth_conf is None:
            raise Exception('Authentication middleware not present')
        if auth_conf.get('setup.enable', True) is True:
            def on_authorize(environ, start_response):
                response = permission.check(request, environ, start_response)
                return response
            # check if the user has been authenticated
            env = request.environ.copy()
            # use ``func`` as the start_response callback
            is_authorized = bool(on_authorize(env, func))
            if is_authorized:
                #print 'User is authorized access to %s' % request.path
                return func(request, *args, **kwargs)
        return func(request, *args, **kwargs)
    return decorator(validate)

def authorize_request(request, permission):
    """
    This function can be used within a generic view to ensure that no code 
    after the function call is executed if the user doesn't pass the permission
    check specified by ``permission``.
    """

    return authkit_authorize_middleware(request, permission)

def authorized(request, permission):
    """
    Similar to the ``authorize_request()`` function with no access to the
    request but rather than raising an exception to stop the request if a
    permission check fails, this function simply returns ``False`` so that you
    can test permissions in your code without triggering a sign in. It can
    therefore be used in a controller action or template.

    Use like this::

        if authorized(permission):
            return Response('You are authorized')
        else:
            return Response('Access denied')
 
    """
    try:
        authorize_request(request, permission)
    except (NotAuthorizedError, NotAuthenticatedError):
        return False
    else:
        return True

